@@ -1,7 +1,7 @@
 ;;; erc-track.el --- Track modified channel buffers
 
 ;; Copyright (C) 2002, 2003, 2004, 2005, 2006,
-;;   2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+;;   2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 
 ;; Author: Mario Lang <mlang@delysid.org>
 ;; Keywords: comm, faces
@@ -649,6 +649,7 @@ module, otherwise the keybindings will not do anything useful."
 	   (add-hook 'erc-send-completed-hook 'erc-user-is-active)
 	   (add-hook 'erc-server-001-functions 'erc-user-is-active))
        (erc-track-add-to-mode-line erc-track-position-in-mode-line)
+       (setq erc-modified-channels-object (erc-modified-channels-object nil))
        (erc-update-mode-line)
        (if (featurep 'xemacs)
 	   (defadvice switch-to-buffer (after erc-update (&rest args) activate)
